<?php

declare(strict_types=1);

use PhpCsFixer\Config;
use PhpCsFixer\Finder;
use XFCsFixer\Fixer\CurlyBracesPositionFixer;
use XFCsFixer\Fixer\ElseIfFixer;
use XFCsFixer\Fixer\GroupedFunctionUseStatementFixer;
use XFCsFixer\Fixer\NoBlankLineAtEofFixer;
use XFCsFixer\Fixer\UppercaseForeachAsFixer;

// rough plan:
// - update syntax to PHP 7.2 across the board (around 2.3.0 RC1?)
// - fix dummy issues: EOL/EOF and whitespace, extra newlines, bad indentation, case colons, statements?
// - fix less dummy issues: parens, maybe braces, function calls, switch fall thrus, control structures, imports, method args?
// - update core syntax to custom PSR2 (easy fixes, then harder ones)
// - update core syntax to custom PSR12 (easy fixes, then harder ones)
// - make a case for introducing this, and just use it myself if they hate it

$finder = Finder::create()
	->in(__DIR__)
	->ignoreVCSIgnored(true)
	->notPath([
		// core directories
		'src/tests/',
		'src/XF/Install/_templates/',

		// core files
		'development/requirements/xenforo_requirements.php',
		'src/XF/Authentication/PasswordHash.php',
		'src/XF/Data/Emoji.php',
		'src/XF/Template/Compiler/Parser.php',
		'phpstan-baseline.php',

		// add-on directories
		'src/addons/', // TODO: revisit this
		'/^src\/addons\/[\w\/]+\/_vendor\//',
		'/^src\/addons\/[\w\/]+\/_output\//',
		'src/addons/XFMG/VideoInfo/',

		// add-on files
		'src/addons/XFDevTools/Legacy/Template/Compiler/Lexer.php',
		'src/addons/XFDevTools/Legacy/Template/Compiler/Parser.php',
	]);

$config = new Config('XF');
$config
	->setFinder($finder)
	->registerCustomFixers([
		new CurlyBracesPositionFixer(),
		new ElseIfFixer(),
		new GroupedFunctionUseStatementFixer(),
		new NoBlankLineAtEofFixer(),
		new UppercaseForeachAsFixer(),
	]);

$rules = [
	// base rulesets
	'@PER-CS1.0' => true,
	'@PHP71Migration' => true,

	// custom fixers
	'XF/curly_braces_position' => true,
	'XF/else_if' => true,
	'XF/grouped_function_use_statement' => true,
	'XF/no_blank_line_at_eof' => true,
	'XF/uppercase_foreach_as' => true,

	// @PSR2 customizations
	'control_structure_continuation_position' => [ // we use allman braces
		'position' => 'next_line',
	],
	'braces_position' => [  // we use allman braces
		'control_structures_opening_brace' => 'next_line_unless_newline_at_signature_end',
	],
	'function_declaration' => [ // we use no space after closure parens
		'closure_function_spacing' => 'none',
		'closure_fn_spacing' => 'none',
	],
];

$risky = false;
if ($risky)
{
	$config->setRiskyAllowed(true);

	$rules = array_merge($rules, [
		'@PER-CS1.0:risky' => true,
		'@PHP71Migration:risky' => true,

		// TODO: move these to relaxed mode?
		// @PHP71Migration:risky customizations
		'random_api_migration' => [ // mt_rand is good where it's at
			'replacements' => [
				'getrandmax' => 'mt_getrandmax',
				'rand' => 'mt_rand',
				'srand' => 'mt_srand',
			],
		],

		// @PHP71Migration:risky disablers
		'declare_strict_types' => false, // we can't use strict types
		'void_return' => false, // we can't add void return types
	]);
}

// relaxed mode should not make significant changes to existing code
$relaxed = true;
if ($relaxed)
{
	$rules = array_merge($rules, [
		// @PSR12/PHP71Migration customizations
		'visibility_required' => [ // we do not specify const visibility -- TODO: revisit this
			'elements' => ['property', 'method'],
		],

		// @PSR2 customizations
		'method_argument_space' => [ // we use inconsistent multline method arguments
			'on_multiline' => 'ignore',
		],

		// @PSR12 disablers
		'single_trait_insert_per_statement' => false, // we use multiple traits per statement in some places

		// @PSR2 disablers
		// easy to fix -- TODO: fix these in existing code
		'control_structure_braces' => false,
		'no_break_comment' => false,
		'switch_case_semicolon_to_colon' => false,
		'braces_position' => false, // we use inline {} for empty bodies in some places
		'function_declaration' => false, // we use single space after closure parens in some places
		'no_multiple_statements_per_line' => false, // we use inline switch cases in some places
		'no_spaces_after_function_name' => false, // we use single space after require in some places TODO: can we enforce consistency?
		'statement_indentation' => false, // we have lingering comments switch statements in some places TODO: can we enforce consistency?

		// @PHP71Migration disablers
		'list_syntax' => false, // TODO: this requires no_spaces_after_function_name to be corrected first
	]);
}

$config
	->setIndent("\t")
	->setRules($rules);

return $config;