<?php

namespace XF\Search\Data;

use XF\Mvc\Entity\Entity;
use XF\Search\MetadataStructure;

use function count, is_array;

abstract class AbstractData
{
	protected $contentType;

	/**
	 * @var \XF\Search\Search
	 */
	protected $searcher;

	public function __construct($contentType, \XF\Search\Search $searcher)
	{
		$this->contentType = $contentType;
		$this->searcher = $searcher;
	}

	abstract public function getIndexData(Entity $entity);
	abstract public function setupMetadataStructure(MetadataStructure $structure);
	abstract public function getResultDate(Entity $entity);
	abstract public function getTemplateData(Entity $entity, array $options = []);

	public function getMetadataStructure()
	{
		$structure = new MetadataStructure();
		$this->setupMetadataStructure($structure);

		return $structure->getFields();
	}

	public function getEntityWith($forView = false)
	{
		return [];
	}

	public function getTemplateName()
	{
		return 'public:search_result_' . $this->contentType;
	}

	public function renderResult(Entity $entity, array $options = [])
	{
		$template = $this->getTemplateName();
		$data = $this->getTemplateData($entity, $options);

		return \XF::app()->templater()->renderTemplate($template, $data);
	}

	public function getSearchableContentTypes()
	{
		return [$this->contentType];
	}

	public function getSearchFormTab()
	{
		return null;
	}

	public function getSectionContext()
	{
		return null;
	}

	public function getTypeFormTemplate()
	{
		return 'public:search_form_' . $this->contentType;
	}

	public function getSearchFormData()
	{
		return [];
	}

	public function applyTypeConstraintsFromInput(\XF\Search\Query\Query $query, \XF\Http\Request $request, array &$urlConstraints)
	{
	}

	public function getGroupByType()
	{
		return null;
	}

	public function getTypeOrder($order)
	{
		return null;
	}

	/**
	 * This allows you to specify constraints to avoid including search results
	 * that will ultimately be filtered out due to permissions. In most cases,
	 * the query should not generally be modified. It is passed in to allow
	 * inspection.
	 *
	 * Note that the returned constraints may apply to other content types, so
	 * you may need to vary the constraints based on `$isOnlyType`. When
	 * `$isOnlyType` is false, you should either compose the constraints in a
	 * "type" constraint or return only "none" constraints on keys that are
	 * unique to the pertinent content types.
	 *
	 * @param bool $isOnlyType True if the search is limited to only this type
	 *
	 * @return list<\XF\Search\Query\MetadataConstraintInterface>
	 */
	public function getTypePermissionConstraints(\XF\Search\Query\Query $query, $isOnlyType)
	{
		return [];
	}

	public function canUseInlineModeration(Entity $entity, &$error = null)
	{
		return false;
	}

	public function canViewContent(Entity $entity, &$error = null)
	{
		if (method_exists($entity, 'canView'))
		{
			return $entity->canView($error);
		}

		throw new \LogicException("Could not determine content viewability; please override");
	}

	public function canIncludeInResults(Entity $entity, array $resultIds)
	{
		return true;
	}

	public function getContent($id, $forView = false)
	{
		return \XF::app()->findByContentType($this->contentType, $id, $this->getEntityWith($forView));
	}

	public function getContentInRange($lastId, $amount, $forView = false)
	{
		$entityId = \XF::app()->getContentTypeFieldValue($this->contentType, 'entity');
		if (!$entityId)
		{
			throw new \LogicException("Content type {$this->contentType} must define an 'entity' value");
		}

		$em = \XF::em();
		$key = $em->getEntityStructure($entityId)->primaryKey;
		if (is_array($key))
		{
			if (count($key) > 1)
			{
				throw new \LogicException("Entity $entityId must only have a single primary key");
			}
			$key = reset($key);
		}

		$finder = $em->getFinder($entityId)->where($key, '>', $lastId)
			->order($key)
			->with($this->getEntityWith($forView));

		return $finder->fetch($amount);
	}

	public function getContentType()
	{
		return $this->contentType;
	}
}